@@ -6,7 +6,7 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2008 Insecure.Com LLC. Nmap is    *
+ * The Nmap Security Scanner is (C) 1996-2009 Insecure.Com LLC. Nmap is    *
  * also a registered trademark of Insecure.Com LLC.  This program is free  *
  * software; you may redistribute and/or modify it under the terms of the  *
  * GNU General Public License as published by the Free Software            *
@@ -25,7 +25,7 @@
  * following:                                                              *
  * o Integrates source code from Nmap                                      *
  * o Reads or includes Nmap copyrighted data files, such as                *
- *   nmap-os-fingerprints or nmap-service-probes.                          *
+ *   nmap-os-db or nmap-service-probes.                                    *
  * o Executes Nmap and parses the results (as opposed to typical shell or  *
  *   execution-menu apps, which simply display raw Nmap output and so are  *
  *   not derivative works.)                                                * 
@@ -34,19 +34,10 @@
  * o Links to a library or executes a program that does any of the above   *
  *                                                                         *
  * The term "Nmap" should be taken to also include any portions or derived *
- * works of Nmap.  This list is not exclusive, but is just meant to        *
- * clarify our interpretation of derived works with some common examples.  *
- * These restrictions only apply when you actually redistribute Nmap.  For *
- * example, nothing stops you from writing and selling a proprietary       *
- * front-end to Nmap.  Just distribute it by itself, and point people to   *
- * http://insecure.org/nmap/ to download Nmap.                             *
- *                                                                         *
- * We don't consider these to be added restrictions on top of the GPL, but *
- * just a clarification of how we interpret "derived works" as it applies  *
- * to our GPL-licensed Nmap product.  This is similar to the way Linus     *
- * Torvalds has announced his interpretation of how "derived works"        *
- * applies to Linux kernel modules.  Our interpretation refers only to     *
- * Nmap - we don't speak for any other GPL products.                       *
+ * works of Nmap.  This list is not exclusive, but is meant to clarify our *
+ * interpretation of derived works with some common examples.  Our         *
+ * interpretation applies only to Nmap--we don't speak for other people's  *
+ * GPL works.                                                              *
  *                                                                         *
  * If you have any questions about the GPL licensing restrictions on using *
  * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
@@ -60,7 +51,7 @@
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
  * permission to link the code of this program with any version of the     *
  * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included Copying.OpenSSL file, and distribute linked      *
+ * listed in the included COPYING.OpenSSL file, and distribute linked      *
  * combinations including the two. You must obey the GNU GPL in all        *
  * respects for all of the code used other than OpenSSL.  If you modify    *
  * this file, you may extend this exception to your version of the file,   *
@@ -77,28 +68,28 @@
  *                                                                         *
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
  * and add new features.  You are highly encouraged to send your changes   *
- * to fyodor@insecure.org for possible incorporation into the main         *
- * distribution.  By sending these changes to Fyodor or one the            *
+ * to nmap-dev@insecure.org for possible incorporation into the main       *
+ * distribution.  By sending these changes to Fyodor or one of the         *
  * Insecure.Org development mailing lists, it is assumed that you are      *
- * offering Fyodor and Insecure.Com LLC the unlimited, non-exclusive right *
- * to reuse, modify, and relicense the code.  Nmap will always be          *
- * available Open Source, but this is important because the inability to   *
- * relicense code has caused devastating problems for other Free Software  *
- * projects (such as KDE and NASM).  We also occasionally relicense the    *
- * code to third parties as discussed above.  If you wish to specify       *
- * special license conditions of your contributions, just say so when you  *
- * send them.                                                              *
+ * offering the Nmap Project (Insecure.Com LLC) the unlimited,             *
+ * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
+ * will always be available Open Source, but this is important because the *
+ * inability to relicense code has caused devastating problems for other   *
+ * Free Software projects (such as KDE and NASM).  We also occasionally    *
+ * relicense the code to third parties as discussed above.  If you wish to *
+ * specify special license conditions of your contributions, just say so   *
+ * when you send them.                                                     *
  *                                                                         *
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
- * General Public License for more details at                              *
- * http://www.gnu.org/copyleft/gpl.html , or in the COPYING file included  *
- * with Nmap.                                                              *
+ * General Public License v2.0 for more details at                         *
+ * http://www.gnu.org/licenses/gpl-2.0.html , or in the COPYING file       *
+ * included with Nmap.                                                     *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: Target.cc 6633 2007-12-22 06:32:03Z fyodor $ */
+/* $Id: Target.cc 13888 2009-06-24 21:35:54Z fyodor $ */
 
 #ifdef WIN32
 #include "nmap_winconfig.h"
@@ -119,12 +110,12 @@ Target::Target() {
 
 void Target::Initialize() {
   hostname = NULL;
+  targetname = NULL;
   memset(&seq, 0, sizeof(seq));
   distance = -1;
-  FPR1 = NULL;
   FPR = NULL;
   osscan_flag = OS_NOTPERF;
-  wierd_responses = flags = 0;
+  weird_responses = flags = 0;
   memset(&to, 0, sizeof(to));
   memset(&targetsock, 0, sizeof(targetsock));
   memset(&sourcesock, 0, sizeof(sourcesock));
@@ -139,10 +130,13 @@ void Target::Initialize() {
   MACaddress_set = SrcMACaddress_set = NextHopMACaddress_set = false;
   htn.msecs_used = 0;
   htn.toclock_running = false;
+  htn.host_start = htn.host_end = 0;
   interface_type = devt_other;
 	devname[0] = '\0';
 	devfullname[0] = '\0';
   state_reason_init(&reason);
+  memset(&pingprobe, 0, sizeof(pingprobe));
+  pingprobe_state = PORT_UNKNOWN;
 }
 
 
@@ -164,17 +158,18 @@ Target::~Target() {
 }
 
 void Target::FreeInternal() {
-
   /* Free the DNS name if we resolved one */
   if (hostname)
     free(hostname);
 
+  if (targetname)
+    free(targetname);
+
   if (nameIPBuf) {
     free(nameIPBuf);
     nameIPBuf = NULL;
   }
 
-  if (FPR1) delete FPR1;
   if (FPR) delete FPR;
 }
 
@@ -182,7 +177,9 @@ void Target::FreeInternal() {
     Called when the IP changes */
 void Target::GenerateIPString() {
   struct sockaddr_in *sin = (struct sockaddr_in *) &targetsock;
+#if HAVE_IPV6
   struct sockaddr_in6 *sin6 = (struct sockaddr_in6 *) &targetsock;
+#endif
 
   if (inet_ntop(sin->sin_family, (sin->sin_family == AF_INET)? 
                 (char *) &sin->sin_addr : 
@@ -221,6 +218,7 @@ void Target::setTargetSockAddr(struct sockaddr_storage *ss, size_t ss_len) {
     /* We had an old target sock, so we better blow away the hostname as
        this one may be new. */
     setHostName(NULL);
+    setTargetName(NULL);
   }
   memcpy(&targetsock, ss, ss_len);
   targetsocklen = ss_len;
@@ -239,7 +237,7 @@ struct in_addr Target::v4host() {
 }
 
 // Returns IPv4 host address or NULL if unavailable.
-const struct in_addr *Target::v4hostip() {
+const struct in_addr *Target::v4hostip() const {
   struct sockaddr_in *sin = (struct sockaddr_in *) &targetsock;
   if (sin->sin_family == AF_INET) {
     return &(sin->sin_addr);
@@ -308,6 +306,16 @@ void Target::setHostName(char *name) {
   }
 }
 
+void Target::setTargetName(char *name) {
+  if (targetname) {
+    free(targetname);
+    targetname = NULL;
+  }
+  if (name) {
+    targetname = strdup(name);
+  }
+}
+
  /* Generates a printable string consisting of the host's IP
      address and hostname (if available).  Eg "www.insecure.org
      (64.71.184.53)" or "fe80::202:e3ff:fe14:1102".  The name is
@@ -376,6 +384,7 @@ void Target::startTimeOutClock(const struct timeval *now) {
   htn.toclock_running = true;
   if (now) htn.toclock_start = *now;
   else gettimeofday(&htn.toclock_start, NULL);
+  if (!htn.host_start) htn.host_start = htn.toclock_start.tv_sec;
 }
   /* The complement to startTimeOutClock. */
 void Target::stopTimeOutClock(const struct timeval *now) {
@@ -385,6 +394,7 @@ void Target::stopTimeOutClock(const struct timeval *now) {
   if (now) tv = *now;
   else gettimeofday(&tv, NULL);
   htn.msecs_used += TIMEVAL_MSEC_SUBTRACT(tv, htn.toclock_start);
+  htn.host_end = tv.tv_sec;
 }
   /* Returns whether the host is timedout.  If the timeoutclock is
      running, counts elapsed time for that.  Pass NULL if you don't have the
@@ -437,7 +447,7 @@ void Target::setDeviceNames(const char *name, const char *fullname) {
 }
 
 /* Returns the 6-byte long MAC address, or NULL if none has been set */
-const u8 *Target::MACAddress() {
+const u8 *Target::MACAddress() const {
   return (MACaddress_set)? MACaddress : NULL;
 }
 
@@ -460,3 +470,4 @@ void Target::osscanSetFlag(int flag) {
 		osscan_flag = flag;
 }
 
+
