@@ -4,7 +4,7 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2008 Insecure.Com LLC. Nmap is    *
+ * The Nmap Security Scanner is (C) 1996-2009 Insecure.Com LLC. Nmap is    *
  * also a registered trademark of Insecure.Com LLC.  This program is free  *
  * software; you may redistribute and/or modify it under the terms of the  *
  * GNU General Public License as published by the Free Software            *
@@ -23,7 +23,7 @@
  * following:                                                              *
  * o Integrates source code from Nmap                                      *
  * o Reads or includes Nmap copyrighted data files, such as                *
- *   nmap-os-fingerprints or nmap-service-probes.                          *
+ *   nmap-os-db or nmap-service-probes.                                    *
  * o Executes Nmap and parses the results (as opposed to typical shell or  *
  *   execution-menu apps, which simply display raw Nmap output and so are  *
  *   not derivative works.)                                                * 
@@ -32,19 +32,10 @@
  * o Links to a library or executes a program that does any of the above   *
  *                                                                         *
  * The term "Nmap" should be taken to also include any portions or derived *
- * works of Nmap.  This list is not exclusive, but is just meant to        *
- * clarify our interpretation of derived works with some common examples.  *
- * These restrictions only apply when you actually redistribute Nmap.  For *
- * example, nothing stops you from writing and selling a proprietary       *
- * front-end to Nmap.  Just distribute it by itself, and point people to   *
- * http://nmap.org to download Nmap.                                       *
- *                                                                         *
- * We don't consider these to be added restrictions on top of the GPL, but *
- * just a clarification of how we interpret "derived works" as it applies  *
- * to our GPL-licensed Nmap product.  This is similar to the way Linus     *
- * Torvalds has announced his interpretation of how "derived works"        *
- * applies to Linux kernel modules.  Our interpretation refers only to     *
- * Nmap - we don't speak for any other GPL products.                       *
+ * works of Nmap.  This list is not exclusive, but is meant to clarify our *
+ * interpretation of derived works with some common examples.  Our         *
+ * interpretation applies only to Nmap--we don't speak for other people's  *
+ * GPL works.                                                              *
  *                                                                         *
  * If you have any questions about the GPL licensing restrictions on using *
  * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
@@ -58,7 +49,7 @@
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
  * permission to link the code of this program with any version of the     *
  * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included Copying.OpenSSL file, and distribute linked      *
+ * listed in the included COPYING.OpenSSL file, and distribute linked      *
  * combinations including the two. You must obey the GNU GPL in all        *
  * respects for all of the code used other than OpenSSL.  If you modify    *
  * this file, you may extend this exception to your version of the file,   *
@@ -75,24 +66,24 @@
  *                                                                         *
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
  * and add new features.  You are highly encouraged to send your changes   *
- * to fyodor@insecure.org for possible incorporation into the main         *
+ * to nmap-dev@insecure.org for possible incorporation into the main       *
  * distribution.  By sending these changes to Fyodor or one of the         *
  * Insecure.Org development mailing lists, it is assumed that you are      *
- * offering Fyodor and Insecure.Com LLC the unlimited, non-exclusive right *
- * to reuse, modify, and relicense the code.  Nmap will always be          *
- * available Open Source, but this is important because the inability to   *
- * relicense code has caused devastating problems for other Free Software  *
- * projects (such as KDE and NASM).  We also occasionally relicense the    *
- * code to third parties as discussed above.  If you wish to specify       *
- * special license conditions of your contributions, just say so when you  *
- * send them.                                                              *
+ * offering the Nmap Project (Insecure.Com LLC) the unlimited,             *
+ * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
+ * will always be available Open Source, but this is important because the *
+ * inability to relicense code has caused devastating problems for other   *
+ * Free Software projects (such as KDE and NASM).  We also occasionally    *
+ * relicense the code to third parties as discussed above.  If you wish to *
+ * specify special license conditions of your contributions, just say so   *
+ * when you send them.                                                     *
  *                                                                         *
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
- * General Public License for more details at                              *
- * http://www.gnu.org/copyleft/gpl.html , or in the COPYING file included  *
- * with Nmap.                                                              *
+ * General Public License v2.0 for more details at                         *
+ * http://www.gnu.org/licenses/gpl-2.0.html , or in the COPYING file       *
+ * included with Nmap.                                                     *
  *                                                                         *
  ***************************************************************************/
 
@@ -240,6 +231,8 @@ static int read_timeouts[][4] = {
 // Size of hash table used to hold the hosts from /etc/hosts
 #define HASH_TABLE_SIZE 256
 
+// Hash macro for etchosts
+#define IP_HASH(x) (ntohl(x)%HASH_TABLE_SIZE)
 
 
 //------------------- Internal Structures ---------------------
@@ -855,6 +848,14 @@ static void connect_dns_servers() {
     s = *serverI;
 
     s->nsd = nsi_new(dnspool, NULL);
+    if (o.spoofsource) {
+      struct sockaddr_storage ss;
+      size_t sslen;
+      o.SourceSockAddr(&ss, &sslen);
+      nsi_set_localaddr(s->nsd, &ss, sslen);
+    }
+    if (o.ipoptionslen)
+      nsi_set_ipoptions(s->nsd, o.ipoptions, o.ipoptionslen);
     s->reqs_on_wire = 0;
     s->capacity = CAPACITY_MIN;
     s->write_busy = 0;
@@ -868,6 +869,8 @@ static void connect_dns_servers() {
 
 
 #ifdef WIN32
+// Reads the Windows registry and adds all the nameservers found via the
+// add_dns_server() function.
 void win32_read_registry(char *controlset) {
   HKEY hKey;
   HKEY hKey2;
@@ -927,12 +930,9 @@ void win32_read_registry(char *controlset) {
 
 
 
-// Parses /etc/resolv.conf (unix) or the registry (win32) and adds
-// all the nameservers found via the add_dns_server() function.
+// Parses /etc/resolv.conf (unix) and adds all the nameservers found via the
+// add_dns_server() function.
 static void parse_resolvdotconf() {
-
-#ifndef WIN32
-
   FILE *fp;
   char buf[2048], *tp;
   char ipaddr[16];
@@ -958,10 +958,6 @@ static void parse_resolvdotconf() {
   }
 
   fclose(fp);
-
-#else
-  win32_read_registry("CurrentControlSet");
-#endif
 }
 
 
@@ -1047,7 +1043,7 @@ static void addto_etchosts(u32 ip, const char *hname) {
   he->name = strdup(hname);
   he->addr = ip;
   he->cache_hits = 0;
-  etchosts[ip % HASH_TABLE_SIZE].push_back(he);
+  etchosts[IP_HASH(ip)].push_back(he);
   total_size++;
 }
 
@@ -1056,8 +1052,8 @@ static void addto_etchosts(u32 ip, const char *hname) {
 static char *lookup_etchosts(u32 ip) {
   std::list<host_elem *>::iterator hostI;
   host_elem *tpelem;
-
-  for(hostI = etchosts[ip % HASH_TABLE_SIZE].begin(); hostI != etchosts[ip % HASH_TABLE_SIZE].end(); hostI++) {
+  int localIP_Hash = IP_HASH(ip);
+  for(hostI = etchosts[localIP_Hash].begin(); hostI != etchosts[localIP_Hash].end(); hostI++) {
     tpelem = *hostI;
     if (tpelem->addr == ip) {
       if(tpelem->cache_hits < UCHAR_MAX)
@@ -1103,6 +1099,31 @@ static void etchosts_init(void) {
 #endif
 }
 
+/* Initialize the global servs list of DNS servers. If the --dns-servers option
+ * was given, use the listed servers; otherwise get the list from resolv.conf or
+ * the Windows registry. If o.mass_dns is false, the list of servers is empty.
+ * This function caches the results from the first time it is run. */
+static void init_servs(void) {
+  static bool initialized = false;
+
+  if (initialized)
+    return;
+
+  initialized = true;
+
+  if (!o.mass_dns)
+    return;
+
+  if (o.dns_servers) {
+    add_dns_server(o.dns_servers);
+  } else {
+#ifndef WIN32
+    parse_resolvdotconf();
+#else
+    win32_read_registry("CurrentControlSet");
+#endif
+  }
+}
 
 //------------------- Main loops ---------------------
 
@@ -1119,51 +1140,10 @@ static void nmap_mass_rdns_core(Target **targets, int num_targets) {
   bool lasttrace = false;
   char spmobuf[1024];
 
-  if (o.mass_dns == false) {
-    Target *currenths;
-    struct sockaddr_storage ss;
-    size_t sslen;
-    char hostname[MAXHOSTNAMELEN + 1] = "";
+  // If necessary, set up the dns server list
+  init_servs();
 
-    for(hostI = targets; hostI < targets+num_targets; hostI++) {
-      currenths = *hostI;
-
-      if (((currenths->flags & HOST_UP) || o.resolve_all) && !o.noresolve) stat_actual++;
-    }
-
-    Snprintf(spmobuf, sizeof(spmobuf), "System DNS resolution of %d host%s.", num_targets, num_targets-1 ? "s" : "");
-    SPM = new ScanProgressMeter(spmobuf);
-
-    for(i=0, hostI = targets; hostI < targets+num_targets; hostI++, i++) {
-      currenths = *hostI;
-	
-      if (keyWasPressed())
-        SPM->printStats((double) i / stat_actual, NULL);
-
-      if (((currenths->flags & HOST_UP) || o.resolve_all) && !o.noresolve) {
-        if (currenths->TargetSockAddr(&ss, &sslen) != 0)
-          fatal("Failed to get target socket address.");
-        if (getnameinfo((struct sockaddr *)&ss, sslen, hostname,
-                        sizeof(hostname), NULL, 0, NI_NAMEREQD) == 0) {
-          stat_ok++;
-          currenths->setHostName(hostname);
-        }
-      }
-    }
-
-    SPM->endTask(NULL, NULL);
-    delete SPM;
-
-    return;
-  }
-
-  // If necessary, set up the dns server list from resolv.conf
-  if (servs.size() == 0) {
-    if (o.dns_servers) add_dns_server(o.dns_servers);
-    else parse_resolvdotconf();
-
-    if (servs.size() == 0 && firstrun) error("mass_dns: warning: Unable to determine any DNS servers. Reverse DNS is disabled. Try using --system-dns or specify valid servers with --dns_servers");
-  }
+  if (servs.size() == 0 && firstrun) error("mass_dns: warning: Unable to determine any DNS servers. Reverse DNS is disabled. Try using --system-dns or specify valid servers with --dns-servers");
 
 
   // If necessary, set up the /etc/hosts hashtable
@@ -1204,7 +1184,7 @@ static void nmap_mass_rdns_core(Target **targets, int num_targets) {
     fatal("Unable to create nsock pool in %s()", __func__);
 
   if ((lasttrace = o.packetTrace()))
-    nsp_settrace(dnspool, 5, o.getStartTime());
+    nsp_settrace(dnspool, NSOCK_TRACE_LEVEL, o.getStartTime());
   
   connect_dns_servers();
 
@@ -1226,7 +1206,7 @@ static void nmap_mass_rdns_core(Target **targets, int num_targets) {
     if (o.packetTrace() != lasttrace) {
       lasttrace = !lasttrace;
       if (lasttrace)
-	nsp_settrace(dnspool, 5, o.getStartTime());
+	nsp_settrace(dnspool, NSOCK_TRACE_LEVEL, o.getStartTime());
       else nsp_settrace(dnspool, 0, o.getStartTime());
     }
     nsock_loop(dnspool, timeout);
@@ -1278,6 +1258,44 @@ static void nmap_mass_rdns_core(Target **targets, int num_targets) {
 
 }
 
+static void nmap_system_rdns_core(Target **targets, int num_targets) {
+  Target **hostI;
+  Target *currenths;
+  struct sockaddr_storage ss;
+  size_t sslen;
+  char hostname[MAXHOSTNAMELEN + 1] = "";
+  char spmobuf[1024];
+  int i;
+
+  for(hostI = targets; hostI < targets+num_targets; hostI++) {
+    currenths = *hostI;
+
+    if (((currenths->flags & HOST_UP) || o.resolve_all) && !o.noresolve) stat_actual++;
+  }
+
+  Snprintf(spmobuf, sizeof(spmobuf), "System DNS resolution of %d host%s.", num_targets, num_targets-1 ? "s" : "");
+  SPM = new ScanProgressMeter(spmobuf);
+
+  for(i=0, hostI = targets; hostI < targets+num_targets; hostI++, i++) {
+    currenths = *hostI;
+      
+    if (keyWasPressed())
+      SPM->printStats((double) i / stat_actual, NULL);
+
+    if (((currenths->flags & HOST_UP) || o.resolve_all) && !o.noresolve) {
+      if (currenths->TargetSockAddr(&ss, &sslen) != 0)
+        fatal("Failed to get target socket address.");
+      if (getnameinfo((struct sockaddr *)&ss, sslen, hostname,
+                      sizeof(hostname), NULL, 0, NI_NAMEREQD) == 0) {
+        stat_ok++;
+        currenths->setHostName(hostname);
+      }
+    }
+  }
+
+  SPM->endTask(NULL, NULL);
+  delete SPM;
+}
 
 
 // Publicly available function. Basically just a wrapper so we
@@ -1290,13 +1308,17 @@ void nmap_mass_rdns(Target **targets, int num_targets) {
 
   stat_actual = stat_ok = stat_nx = stat_sf = stat_trans = stat_dropped = stat_cname = 0;
 
-  nmap_mass_rdns_core(targets, num_targets);
+  // mass_dns only supports IPv4.
+  if (o.mass_dns && o.af() == AF_INET)
+    nmap_mass_rdns_core(targets, num_targets);
+  else
+    nmap_system_rdns_core(targets, num_targets);
 
   gettimeofday(&now, NULL);
 
   if (stat_actual > 0) {
     if (o.debugging || o.verbose >= 3) {
-      if (o.mass_dns) {
+      if (o.mass_dns && o.af() == AF_INET) {
 	// #:  Number of DNS servers used
 	// OK: Number of fully reverse resolved queries
 	// NX: Number of confirmations of 'No such reverse domain eXists'
@@ -1316,3 +1338,20 @@ void nmap_mass_rdns(Target **targets, int num_targets) {
 
   firstrun=0;
 }
+
+
+// Returns a list of known DNS servers
+std::list<std::string> get_dns_servers() {
+  init_servs();
+
+  // If the user said --system-dns (!o.mass_dns), we should never return a list
+  // of servers.
+  assert(o.mass_dns || servs.empty());
+
+  std::list<dns_server *>::iterator servI;
+  std::list<std::string> serverList;
+  for(servI = servs.begin(); servI != servs.end(); servI++)
+    serverList.push_back(inet_ntoa((*servI)->addr.sin_addr));
+
+  return serverList;
+}
